# -*- coding: UTF-8 -*-
from __future__ import absolute_import, unicode_literals

# Standard Library
import codecs
import os
from builtins import bytes  # noqa
from builtins import str  # noqa

# AMP Renderer
from amp_renderer import AMPRenderer

# Current AMP runtime style
AMP_RUNTIME_VERSION = '012008150009001'
AMP_RUNTIME_STYLES = """html{overflow-x:hidden!important}html.i-amphtml-fie{height:100%!important;width:100%!important}html:not([amp4ads]),html:not([amp4ads]) body{height:auto!important}html:not([amp4ads]) body{margin:0!important}body{-webkit-text-size-adjust:100%;-moz-text-size-adjust:100%;-ms-text-size-adjust:100%;text-size-adjust:100%}html.i-amphtml-singledoc.i-amphtml-embedded{-ms-touch-action:pan-y;touch-action:pan-y}html.i-amphtml-fie>body,html.i-amphtml-singledoc>body{overflow:visible!important}html.i-amphtml-fie:not(.i-amphtml-inabox)>body,html.i-amphtml-singledoc:not(.i-amphtml-inabox)>body{position:relative!important}html.i-amphtml-webview>body{overflow-x:hidden!important;overflow-y:visible!important;min-height:100vh!important}html.i-amphtml-ios-embed-legacy>body{overflow-x:hidden!important;overflow-y:auto!important;position:absolute!important}html.i-amphtml-ios-embed{overflow-y:auto!important;position:static}#i-amphtml-wrapper{overflow-x:hidden!important;overflow-y:auto!important;position:absolute!important;top:0!important;left:0!important;right:0!important;bottom:0!important;margin:0!important;display:block!important}html.i-amphtml-ios-embed.i-amphtml-ios-overscroll,html.i-amphtml-ios-embed.i-amphtml-ios-overscroll>#i-amphtml-wrapper{-webkit-overflow-scrolling:touch!important}#i-amphtml-wrapper>body{position:relative!important;border-top:1px solid transparent!important}#i-amphtml-wrapper+body{visibility:visible}#i-amphtml-wrapper+body .i-amphtml-lightbox-element,#i-amphtml-wrapper+body[i-amphtml-lightbox]{visibility:hidden}#i-amphtml-wrapper+body[i-amphtml-lightbox] .i-amphtml-lightbox-element{visibility:visible}#i-amphtml-wrapper.i-amphtml-scroll-disabled,.i-amphtml-scroll-disabled{overflow-x:hidden!important;overflow-y:hidden!important}amp-instagram{padding:54px 0px 0px!important;background-color:#fff}amp-iframe iframe{box-sizing:border-box!important}[amp-access][amp-access-hide]{display:none}[subscriptions-dialog],body:not(.i-amphtml-subs-ready) [subscriptions-action],body:not(.i-amphtml-subs-ready) [subscriptions-section]{display:none!important}amp-experiment,amp-live-list>[update]{display:none}.i-amphtml-jank-meter{position:fixed;background-color:rgba(232,72,95,0.5);bottom:0;right:0;color:#fff;font-size:16px;z-index:1000;padding:5px}amp-list[resizable-children]>.i-amphtml-loading-container.amp-hidden{display:none!important}amp-list [fetch-error],amp-list[load-more] [load-more-button],amp-list[load-more] [load-more-end],amp-list[load-more] [load-more-failed],amp-list[load-more] [load-more-loading]{display:none}amp-list[diffable] div[role=list]{display:block}amp-story-page,amp-story[standalone]{min-height:1px!important;display:block!important;height:100%!important;margin:0!important;padding:0!important;overflow:hidden!important;width:100%!important}amp-story[standalone]{background-color:#202125!important;position:relative!important}amp-story-page{background-color:#757575}amp-story .amp-active>div,amp-story .i-amphtml-loader-background{display:none!important}amp-story-page:not(:first-of-type):not([distance]):not([active]){transform:translateY(1000vh)!important}amp-autocomplete{position:relative!important;display:inline-block!important}amp-autocomplete>input,amp-autocomplete>textarea{padding:0.5rem;border:1px solid rgba(0,0,0,0.33)}.i-amphtml-autocomplete-results,amp-autocomplete>input,amp-autocomplete>textarea{font-size:1rem;line-height:1.5rem}[amp-fx^=fly-in]{visibility:hidden}amp-script[nodom]{position:fixed!important;top:0!important;width:1px!important;height:1px!important;overflow:hidden!important;visibility:hidden}
/*# sourceURL=/css/ampdoc.css*/[hidden]{display:none!important}.i-amphtml-element{display:inline-block}.i-amphtml-blurry-placeholder{transition:opacity 0.3s cubic-bezier(0.0,0.0,0.2,1)!important;pointer-events:none}[layout=nodisplay]:not(.i-amphtml-element){display:none!important}.i-amphtml-layout-fixed,[layout=fixed][width][height]:not(.i-amphtml-layout-fixed){display:inline-block;position:relative}.i-amphtml-layout-responsive,[layout=responsive][width][height]:not(.i-amphtml-layout-responsive),[width][height][heights]:not([layout]):not(.i-amphtml-layout-responsive),[width][height][sizes]:not([layout]):not(.i-amphtml-layout-responsive){display:block;position:relative}.i-amphtml-layout-intrinsic,[layout=intrinsic][width][height]:not(.i-amphtml-layout-intrinsic){display:inline-block;position:relative;max-width:100%}.i-amphtml-layout-intrinsic .i-amphtml-sizer{max-width:100%}.i-amphtml-intrinsic-sizer{max-width:100%;display:block!important}.i-amphtml-layout-container,.i-amphtml-layout-fixed-height,[layout=container],[layout=fixed-height][height]:not(.i-amphtml-layout-fixed-height){display:block;position:relative}.i-amphtml-layout-fill,[layout=fill]:not(.i-amphtml-layout-fill){display:block;overflow:hidden!important;position:absolute;top:0;left:0;bottom:0;right:0}.i-amphtml-layout-flex-item,[layout=flex-item]:not(.i-amphtml-layout-flex-item){display:block;position:relative;-ms-flex:1 1 auto;flex:1 1 auto}.i-amphtml-layout-fluid{position:relative}.i-amphtml-layout-size-defined{overflow:hidden!important}.i-amphtml-layout-awaiting-size{position:absolute!important;top:auto!important;bottom:auto!important}i-amphtml-sizer{display:block!important}.i-amphtml-blurry-placeholder,.i-amphtml-fill-content{display:block;height:0;max-height:100%;max-width:100%;min-height:100%;min-width:100%;width:0;margin:auto}.i-amphtml-layout-size-defined .i-amphtml-fill-content{position:absolute;top:0;left:0;bottom:0;right:0}.i-amphtml-replaced-content,.i-amphtml-screen-reader{padding:0!important;border:none!important}.i-amphtml-screen-reader{position:fixed!important;top:0px!important;left:0px!important;width:4px!important;height:4px!important;opacity:0!important;overflow:hidden!important;margin:0!important;display:block!important;visibility:visible!important}.i-amphtml-screen-reader~.i-amphtml-screen-reader{left:8px!important}.i-amphtml-screen-reader~.i-amphtml-screen-reader~.i-amphtml-screen-reader{left:12px!important}.i-amphtml-screen-reader~.i-amphtml-screen-reader~.i-amphtml-screen-reader~.i-amphtml-screen-reader{left:16px!important}.i-amphtml-unresolved{position:relative;overflow:hidden!important}.i-amphtml-select-disabled{-webkit-user-select:none!important;-moz-user-select:none!important;-ms-user-select:none!important;user-select:none!important}.i-amphtml-notbuilt,[layout]:not(.i-amphtml-element),[width][height][heights]:not([layout]):not(.i-amphtml-element),[width][height][sizes]:not([layout]):not(.i-amphtml-element){position:relative;overflow:hidden!important;color:transparent!important}.i-amphtml-notbuilt:not(.i-amphtml-layout-container)>*,[layout]:not([layout=container]):not(.i-amphtml-element)>*,[width][height][heights]:not([layout]):not(.i-amphtml-element)>*,[width][height][sizes]:not([layout]):not(.i-amphtml-element)>*{display:none}.i-amphtml-notbuilt:not(.i-amphtml-layout-container),[layout]:not([layout=container]):not(.i-amphtml-element),[width][height][heights]:not([layout]):not(.i-amphtml-element),[width][height][sizes]:not([layout]):not(.i-amphtml-element){color:transparent!important;line-height:0!important}.i-amphtml-ghost{visibility:hidden!important}.i-amphtml-element>[placeholder],[layout]:not(.i-amphtml-element)>[placeholder],[width][height][heights]:not([layout]):not(.i-amphtml-element)>[placeholder],[width][height][sizes]:not([layout]):not(.i-amphtml-element)>[placeholder]{display:block}.i-amphtml-element>[placeholder].amp-hidden,.i-amphtml-element>[placeholder].hidden{visibility:hidden}.i-amphtml-element:not(.amp-notsupported)>[fallback],.i-amphtml-layout-container>[placeholder].amp-hidden,.i-amphtml-layout-container>[placeholder].hidden{display:none}.i-amphtml-layout-size-defined>[fallback],.i-amphtml-layout-size-defined>[placeholder]{position:absolute!important;top:0!important;left:0!important;right:0!important;bottom:0!important;z-index:1}.i-amphtml-notbuilt>[placeholder]{display:block!important}.i-amphtml-hidden-by-media-query{display:none!important}.i-amphtml-element-error{background:red!important;color:#fff!important;position:relative!important}.i-amphtml-element-error:before{content:attr(error-message)}i-amp-scroll-container,i-amphtml-scroll-container{position:absolute;top:0;left:0;right:0;bottom:0;display:block}i-amp-scroll-container.amp-active,i-amphtml-scroll-container.amp-active{overflow:auto;-webkit-overflow-scrolling:touch}.i-amphtml-loading-container{display:block!important;pointer-events:none;z-index:1}.i-amphtml-notbuilt>.i-amphtml-loading-container{display:block!important}.i-amphtml-loading-container.amp-hidden{visibility:hidden}.i-amphtml-element>[overflow]{cursor:pointer;position:relative;z-index:2;visibility:hidden;display:initial;line-height:normal}.i-amphtml-element>[overflow].amp-visible{visibility:visible}template{display:none!important}.amp-border-box,.amp-border-box *,.amp-border-box :after,.amp-border-box :before{box-sizing:border-box}amp-pixel{display:none!important}amp-analytics,amp-auto-ads,amp-story-auto-ads{position:fixed!important;top:0!important;width:1px!important;height:1px!important;overflow:hidden!important;visibility:hidden}html.i-amphtml-fie>amp-analytics{position:initial!important}[visible-when-invalid]:not(.visible),form [submit-error],form [submit-success],form [submitting]{display:none}amp-accordion{display:block!important}amp-accordion>section{float:none!important}amp-accordion>section>*{float:none!important;display:block!important;overflow:hidden!important;position:relative!important}amp-accordion,amp-accordion>section{margin:0}amp-accordion>section>:last-child{display:none!important}amp-accordion>section[expanded]>:last-child{display:block!important}
/*# sourceURL=/css/ampshared.css*/"""  # noqa


class TestSpeed:
    def test_speed(self):
        local_path = os.path.dirname(__file__)
        input_path = '{}/speed/input.html'.format(local_path)

        with codecs.open(input_path, 'r', encoding='utf-8') as html_file:
            html = html_file.read()

        renderer = AMPRenderer(runtime_version=AMP_RUNTIME_VERSION, runtime_styles=AMP_RUNTIME_STYLES)
        renderer.render(html)

        """This test is for checking large inputs. Replace the contents of
        <input_path>/speed/input.html with a very large HTML file; The test can
        then be profiled with pytest."""

        assert True



# -*- coding: UTF-8 -*-
from __future__ import absolute_import, unicode_literals

# Standard Library
from builtins import bytes  # noqa
from builtins import str  # noqa

# AMP Renderer
from amp_renderer import AMPNode, Layout


class TestLayout:
    def test_get_class(self):
        for layout in list(Layout):
            assert layout.get_class() == 'i-amphtml-layout-{}'.format(layout.value)

    def test_is_size_defined(self):
        size_defined_layouts = [
            Layout.FIXED,
            Layout.FIXED_HEIGHT,
            Layout.RESPONSIVE,
            Layout.FILL,
            Layout.FLEX_ITEM,
            Layout.INTRINSIC,
        ]
        for layout in list(Layout):
            assert layout.is_size_defined() == (layout in size_defined_layouts)


class TestNode:
    ID = 'testID'
    STYLE = 'test:style;'
    CLASSES = ['class1', 'class2']
    DUMMY = 'dummy'

    def _get_attr_value_or_fail(self, attrs, name):
        matches = [x[1] for x in attrs if x[0] == name]
        assert len(matches) == 1
        return matches[0]

    def test_1(self):
        tag = 'amp-test'
        attrs = [
            ('hidden', 'hidden'),
            ('id', self.ID),
            ('style', self.STYLE),
            ('class', ' '.join(self.CLASSES)),
            ('data-dummy', self.DUMMY),
        ]

        node = AMPNode(tag, attrs)

        assert node.id == self.ID
        assert node.tag == tag

        assert set(node.get_attrs()) == set(attrs)

        transformations = node.transform(1)
        assert transformations is None

        transformed_attrs = node.get_attrs()
        assert len(transformed_attrs) == 6

        class_name = self._get_attr_value_or_fail(transformed_attrs, 'class')
        assert set(class_name.split(' ')) == set(self.CLASSES + ['i-amphtml-layout-container'])

        layout = self._get_attr_value_or_fail(transformed_attrs, 'i-amphtml-layout')
        assert layout == Layout.CONTAINER.value

    def test_2(self):
        tag = 'amp-test'
        attrs = [
            ('style', self.STYLE),
            ('class', ' '.join(self.CLASSES)),
            ('layout', 'fixed'),
            ('width', '113'),
            ('height', '140px'),
            ('data-dummy', self.DUMMY),
        ]

        node = AMPNode(tag, attrs)

        assert node.tag == tag
        assert node.id is None

        assert set(node.get_attrs()) == set(attrs)

        transformations = node.transform(1)
        assert transformations is None
        assert node.id is None

        transformed_attrs = node.get_attrs()
        assert len(transformed_attrs) == 7

        class_name = self._get_attr_value_or_fail(transformed_attrs, 'class')
        assert set(class_name.split(' ')) == set(self.CLASSES + [
            'i-amphtml-layout-fixed',
            'i-amphtml-layout-size-defined',
        ])

        layout = self._get_attr_value_or_fail(transformed_attrs, 'i-amphtml-layout')
        assert layout == Layout.FIXED.value

    def test_3(self):
        tag = 'amp-test'
        attrs = [
            ('style', self.STYLE),
            ('class', ' '.join(self.CLASSES)),
            ('layout', 'responsive'),
            ('width', '113'),
            ('height', 'auto'),
            ('media', '(max-width:1024px)'),
            ('srcset', 'https://example.com/1x 133w, https://example.com/2x 266w'),
            ('sizes', '(max-width:533px) 133px, 100vw'),
            ('data-dummy', self.DUMMY),
        ]

        node = AMPNode(tag, attrs)

        assert node.tag == tag

        assert set(node.get_attrs()) == set(attrs)

        transformations = node.transform('i-amp-1')
        assert node.id == 'i-amp-1'
        assert transformations[1]

        transformed_attrs = node.get_attrs()
        assert len(transformed_attrs) == 9

        class_name = self._get_attr_value_or_fail(transformed_attrs, 'class')
        assert set(class_name.split(' ')) == set(self.CLASSES + [
            'i-amphtml-layout-responsive',
            'i-amphtml-layout-size-defined',
        ])

        layout = self._get_attr_value_or_fail(transformed_attrs, 'i-amphtml-layout')
        assert layout == Layout.RESPONSIVE.value



# -*- coding: UTF-8 -*-
from __future__ import absolute_import, unicode_literals

# Standard Library
import json
import re
import sys
from builtins import bytes  # noqa
from builtins import str  # noqa
from collections import OrderedDict, namedtuple
from enum import Enum

if sys.version_info[0] < 3:
    # Third Party
    from HTMLParser import HTMLParser

else:
    # Standard Library
    from html.parser import HTMLParser


class TransformationError(Exception):
    pass


# The namedtuples will still be accessed using index notation for performance.
Translation = namedtuple('Translation', 'selector statements')
Sizes = namedtuple('Size', 'default other')
Media = namedtuple('Media', 'query value')
Sizer = namedtuple('Sizer', 'attrs maybe_img_attrs')


class Translator:
    @classmethod
    def parse_sizes(cls, value):
        # Normalize whitespace
        size_values = re.split(r'\s*,\s*', value.strip())

        try:
            default = size_values.pop()
        except IndexError:
            return ''

        other = []

        for size_value in size_values:
            size_parts = re.split(r'\)\s+', size_value)
            if len(size_parts) != 2:
                raise ValueError('Invalid sizes definition')

            query = '%s)' % size_parts[0]
            query = query.replace(r'\s+', '')
            value = size_parts[1]

            media = Media(query=query, value=value)
            other.append(media)

        return Sizes(default=default, other=other)


class MediaTranslator(Translator):
    @classmethod
    def translate(cls, value, element_id):
        # Normalize whitespace
        media = re.sub(r'\s+', ' ', value).strip()

        if not media:
            return None

        if media[0] == '(':
            media = 'all and %s' % media

        if media.startswith('not '):
            media = media[4:]
        else:
            media = 'not %s' % media

        selector = '#%s' % element_id

        return Translation(
            selector=selector,
            statements=OrderedDict([
                (media, 'display:none'),
            ]))


class SizesTranslator(Translator):
    @classmethod
    def translate(cls, value, element_id):
        sizes = cls.parse_sizes(value)

        selector = '#%s' % element_id

        statements = [(None, 'width:%s' % sizes[0])]

        other_sizes = sizes[1]

        """The user agent will pick a width from the sizes attribute,
        using the first item with a <media-condition> (the part in
        parentheses) that evaluates to true. This means, we have to
        reverse the order the media queries in CSS to emulate this
        behavior (the last definition has precedence)."""
        other_sizes.reverse()

        for size in other_sizes:
            statements.append((size[0], 'width:%s' % size[1]))

        return Translation(
            selector=selector,
            statements=OrderedDict(statements))


class HeightsTranslator(Translator):
    @classmethod
    def translate(cls, value, element_id):
        sizes = cls.parse_sizes(value)

        selector = '#%s>:first-child' % element_id

        statements = [(None, 'padding-top:%s' % sizes[0])]

        other_sizes = sizes[1]

        """The user agent will pick a value from the heights attribute,
        using the first item with a <media-condition> (the part in
        parentheses) that evaluates to true. This means, we have to
        reverse the order the media queries in CSS to emulate this
        behavior (the last definition has precedence)."""
        other_sizes.reverse()
        for size in other_sizes:
            statements.append((size[0], 'padding-top:%s' % size[1]))

        return Translation(
            selector=selector,
            statements=OrderedDict(statements))


TRANSLATIONS = {
    'media': MediaTranslator,
    'sizes': SizesTranslator,
    'heights': HeightsTranslator,
}


class Layout(Enum):
    NODISPLAY = 'nodisplay'
    FIXED = 'fixed'
    FIXED_HEIGHT = 'fixed-height'
    RESPONSIVE = 'responsive'
    CONTAINER = 'container'
    FILL = 'fill'
    FLEX_ITEM = 'flex-item'
    INTRINSIC = 'intrinsic'

    def get_class(self):
        return 'i-amphtml-layout-%s' % self.value

    def is_size_defined(self):
        return self in SIZE_DEFINED_LAYOUTS


# Store constants for performance
LAYOUT_NODISPLAY = Layout.NODISPLAY
LAYOUT_FIXED = Layout.FIXED
LAYOUT_FIXED_HEIGHT = Layout.FIXED_HEIGHT
LAYOUT_RESPONSIVE = Layout.RESPONSIVE
LAYOUT_CONTAINER = Layout.CONTAINER
LAYOUT_FILL = Layout.FILL
LAYOUT_FLEX_ITEM = Layout.FLEX_ITEM
LAYOUT_INTRINSIC = Layout.INTRINSIC

SIZE_DEFINED_LAYOUTS = [
    LAYOUT_FIXED,
    LAYOUT_FIXED_HEIGHT,
    LAYOUT_RESPONSIVE,
    LAYOUT_FILL,
    LAYOUT_FLEX_ITEM,
    LAYOUT_INTRINSIC,
]


class CSSLengthUnit(Enum):
    PX = 'px'
    EM = 'em'
    REM = 'rem'
    VH = 'vh'
    VW = 'vw'
    VMIN = 'vmin'
    VMAX = 'vmax'


# Store constant for performance
UNIT_PX = CSSLengthUnit.PX

# The namedtuples will still be accessed using index notation for performance.
CSSLength = namedtuple('CSSLength', 'numeral unit')
CSS_LENGTH_AUTO = 'auto'
CSS_LENGTH_ONE_PX = CSSLength(numeral=1, unit=UNIT_PX)


class AMPNode:
    def __init__(self, tag, attrs):
        self.tag = tag

        self.id = None

        self._classes = []
        self._style = ''
        self._is_hidden = False

        self._other_attrs = {}
        self._is_transformed = False

        self.sizer = None
        self.maybe_img_attrs = None

        self.should_strip_translated_attrs = True

        for attr in attrs:
            if attr[0] == 'id':
                self.id = attr[1]
            elif attr[0] == 'class':
                self._classes = attr[1].split(' ')
            elif attr[0] == 'style':
                self._style = attr[1]
            elif attr[0] == 'hidden':
                self._is_hidden = True
            else:
                # Should be only one value per key
                self._other_attrs[attr[0]] = attr[1]

    def _parse_length(self, length):
        """Parse a valid length value.

        Returns a CSSLength, or an alternative constant (CSS_LENGTH_AUTO),
        or None; All are valid results.

        Throws a TransformationError if a valid result can't be parsed.

        This utility function is stateless.
        """
        if not length:
            return None

        if length == 'auto':
            return CSS_LENGTH_AUTO

        try:
            match = re.findall(r'(\d+(?:\.\d+)?)(.*)', length)[0]
            numeral = float(match[0])
        except (IndexError, ValueError):
            raise TransformationError('Invalid size value')

        unit_value = match[1] or 'px'
        try:
            unit = CSSLengthUnit(unit_value)
        except ValueError:
            raise TransformationError('Invalid size value')

        return CSSLength(numeral=numeral, unit=unit)

    def transform(self, next_auto_id):
        """Apply the transformation.

        Returns styles that need to be appended to the beginning of the
        amp-custom style section.
        """
        # Create img if necessary
        if self.tag == 'amp-img' and 'data-hero' in self._other_attrs:
            self._other_attrs['i-amphtml-ssr'] = None
            img_attrs = [
                ('class', 'i-amphtml-fill-content i-amphtml-replaced-content'),
                ('decoding', 'async'),
            ]

            attrs_to_copy = [
                'alt',
                'attribution',
                'object-fit',
                'object-position',
                'referrerpolicy',
                'src',
                'srcset',
                'sizes',
                'title',
            ]
            for name in [k for k in attrs_to_copy if k in self._other_attrs]:
                img_attrs.append((name, self._other_attrs[name]))

            self.maybe_img_attrs = img_attrs

        # Translate special attributes to amp-custom CSS
        css_data = None
        did_strip_sizes = False

        translations = [k for k in self._other_attrs if k in TRANSLATIONS]
        if translations:
            potential_id = self.id or next_auto_id

            css_data_items = []
            for t in translations:
                if t == 'sizes' and 'disable-inline-width' in self._other_attrs:
                    """Sizes is meant to be passed to the img tag for source
                    selection, but the user didn’t intend for it to have any
                    side effects. Don’t add any.

                    https://amp.dev/documentation/guides-and-tutorials/learn/amp-html-layout/?format=websites  # noqa
                    """
                    continue

                attribute_value = self._other_attrs[t]
                Translator = TRANSLATIONS[t]  # noqa

                try:
                    translation = Translator.translate(attribute_value, potential_id)
                except ValueError:
                    raise TransformationError('Invalid value for `%s` attribute' % t)
                else:
                    if translation:
                        css_data_items.append(translation)
                        if t == 'sizes':
                            # Need to know so we can apply "responsive" layout
                            did_strip_sizes = True

            if css_data_items:
                used_auto_id = False
                if not self.id:
                    used_auto_id = True
                    self.id = potential_id

                css_data = css_data_items, used_auto_id

            if self.should_strip_translated_attrs:
                for t in translations:
                    if not (t == 'sizes' and not did_strip_sizes):
                        del self._other_attrs[t]

        # Apply the transformation
        layout_value = self._other_attrs.get('layout')

        width = self._parse_length(self._other_attrs.get('width'))
        if not isinstance(width, CSSLength) and layout_value in [None, 'fixed']:
            try:
                width = {
                    'amp-analytics': CSS_LENGTH_ONE_PX,
                    'amp-audio': CSS_LENGTH_AUTO,
                    'amp-pixel': CSS_LENGTH_ONE_PX,
                    'amp-social-share': CSSLength(numeral=60, unit=UNIT_PX),
                }[self.tag]
            except KeyError:
                pass

        height = self._parse_length(self._other_attrs.get('height'))
        if not isinstance(height, CSSLength) and layout_value in [None, 'fixed', 'fixed-height']:
            try:
                height = {
                    'amp-analytics': CSS_LENGTH_ONE_PX,
                    'amp-audio': CSS_LENGTH_AUTO,
                    'amp-pixel': CSS_LENGTH_ONE_PX,
                    'amp-social-share': CSSLength(numeral=44, unit=UNIT_PX),
                }[self.tag]
            except KeyError:
                pass

        if not layout_value:
            width_is_set = isinstance(width, CSSLength)
            height_is_set = isinstance(height, CSSLength)

            if not any([width_is_set, height_is_set]):
                layout_value = 'container'
            elif height_is_set and not width_is_set:
                layout_value = 'fixed_height'
            elif height_is_set and width_is_set and did_strip_sizes:
                layout_value = 'responsive'

                """Apply this directly, because otherwise the runtime won't see
                the `sizes` attribute, and will think the element should be
                "fixed"."""
                self._other_attrs['layout'] = layout_value
            else:
                layout_value = 'fixed'

        try:
            layout = Layout(layout_value)
        except ValueError:
            raise TransformationError('Transformation not supported')

        self._classes.append(layout.get_class())
        if layout.is_size_defined():
            self._classes.append('i-amphtml-layout-size-defined')

        if layout == LAYOUT_NODISPLAY:
            self._is_hidden = True

        elif layout == LAYOUT_FIXED:
            if not all(isinstance(length, CSSLength) for length in [width, height]):
                raise TransformationError('Length and width required for fixed layout')

            self._style = 'width:{}{};height:{}{};{}'.format(
                str(width[0]).rstrip('0').rstrip('.'),
                width[1].value,
                str(height[0]).rstrip('0').rstrip('.'),
                height[1].value,
                self._style)

        elif layout == LAYOUT_FIXED_HEIGHT:
            if not isinstance(height, CSSLength):
                raise TransformationError('Length and width required for fixed layout')

            self._style = 'height:{}{};{}'.format(
                str(height[0]).rstrip('0').rstrip('.'),
                height[1].value,
                self._style)

        elif layout == LAYOUT_FLEX_ITEM:
            if isinstance(height, CSSLength):
                self._style = 'height:{}{};{}'.format(
                    str(height[0]).rstrip('0').rstrip('.'),
                    height[1].value,
                    self._style)

            if isinstance(width, CSSLength):
                self._style = 'width:{}{};{}'.format(
                    str(width[0]).rstrip('0').rstrip('.'),
                    width[1].value,
                    self._style)

        self._other_attrs['i-amphtml-layout'] = layout.value

        # Create sizer if necessary
        if all(isinstance(length, CSSLength) for length in [width, height]):
            if all([width[0] != 0, width[1] == height[1]]):
                if layout == LAYOUT_RESPONSIVE:
                    padding = (height[0] / width[0]) * 100
                    style = 'display:block;padding-top:{:.4f}%;'.format(padding)
                    self.sizer = Sizer(attrs=[('style', style)], maybe_img_attrs=None)

                elif layout == LAYOUT_INTRINSIC:
                    svg_string = '<svg height="{}" width="{}" xmlns="http://www.w3.org/2000/svg" version="1.1"/>'
                    svg_string = svg_string.format(height[0], width[0])

                    img_attrs = [
                        ('alt', ''),
                        ('aria-hidden', 'true'),
                        ('class', 'i-amphtml-intrinsic-sizer'),
                        ('role', 'presentation'),
                        ('src', 'data:image/svg+xml;charset=utf-8,%s' % svg_string),
                    ]

                    self.sizer = Sizer(attrs=[('class', 'i-amphtml-sizer')], maybe_img_attrs=img_attrs)

        return css_data

    def get_attrs(self):
        """Return an list of attribute tuples that represents current state.

        This returns different values before and after a call to `transform()`.
        """
        attrs = list(self._other_attrs.items())

        if self.id:
            attrs.insert(0, ('id', self.id))

        if self._classes:
            attrs.insert(0, ('class', ' '.join(self._classes)))

        if self._style:
            attrs.insert(0, ('style', self._style))

        if self._is_hidden:
            attrs.append(('hidden', 'hidden'))

        return attrs


class AMPRenderer(HTMLParser, object):
    """A parser to ingest AMP HTML and perform various transformations."""

    RENDER_DELAYING_EXTENSIONS = [
        'amp-dynamic-css-classes',
        # 'amp-experiment',  # we handle this by looking for the tag instead
        'amp-story',
    ]

    ID_PREFIX = 'i-amp-'

    def __init__(self, runtime_styles, runtime_version, *args, **kwargs):
        """Initialize AMPRenderer with runtime styles & version.

        Parameters:
            runtime_styles (string): The current contents of
                                     https://cdn.ampproject.org/v0.css

            runtime_version (string): The version number for the runtime
                                      styles as a string with leading zeros,
                                      e.g. '012007302351001'

        """
        super(AMPRenderer, self).__init__(*args, **kwargs)

        self.runtime_styles = runtime_styles
        self.runtime_version = runtime_version

        self._is_test_mode = False
        if not self.runtime_styles or not self.runtime_version:
            self._is_test_mode = True

        self.should_trim_attrs = False
        self.should_strip_comments = False

        # Always keep charrefs intact; This class is meant to reproduce HTML.
        self.convert_charrefs = False

    def reset(self):
        super(AMPRenderer, self).reset()

        self._should_remove_boilerplate = True

        self._boilerplate = ''
        self._is_in_boilerplate = False
        self._boilerplate_index = None

        self._noscript_boilerplate = ''
        self._is_in_noscript = False
        self._noscript_boilerplate_index = None

        self._is_expecting_experiment_script = False
        self._is_expecting_experiment_data = False
        self._is_expecting_experiment_end = False
        self._current_experiment_data = ''

        self._is_render_paused = False
        self._is_render_cancelled = False

        self._result = []
        self._found_custom_element = False
        self._next_auto_id_num = 0
        self._translated_css_data = []

        self._translated_styles_index = None

        try:
            del self.no_boilerplate
        except AttributeError:
            pass

    def _get_next_auto_id(self):
        return self.ID_PREFIX + str(self._next_auto_id_num)

    def _increment_auto_id_num(self):
        self._next_auto_id_num += 1
        while self._next_auto_id_num in self._auto_id_nums_to_ignore:
            self._next_auto_id_num += 1

    def _apply_experiment_data(self):
        self._result.append(self._current_experiment_data)

        experiment_data = self._current_experiment_data

        self._current_experiment_data = ''
        self._is_expecting_experiment_data = False

        return experiment_data

    def handle_decl(self, decl):
        self._result.append('<!%s>' % decl.lower())

    def handle_starttag(self, tag, attrs):
        tag = tag.lower()

        if self._is_expecting_experiment_data:
            # Expecting some JSON, found a start tag == NO EXPERIMENT
            self._apply_experiment_data()

        if self._is_expecting_experiment_script:
            self._is_expecting_experiment_script = False

            if tag == 'script':
                for attr in attrs:
                    if attr[0] == 'type' and attr[1] == 'application/json':
                        """Expecting script & found it; Next, we expect some
                        JSON data"""
                        self._is_expecting_experiment_data = True

        if self._is_expecting_experiment_end:
            # Expecting </amp-experiment>, found a start tag == NO EXPERIMENT
            self._is_expecting_experiment_end = False

        if tag == 'noscript':
            self._is_in_noscript = True

        if tag == 'style':
            if 'amp-boilerplate' in (attr[0] for attr in attrs):
                self._is_in_boilerplate = True

                # Add appropriate boilerplate placeholder
                if self._is_in_noscript and self._noscript_boilerplate_index is None:
                    self._noscript_boilerplate_index = len(self._result)
                elif self._boilerplate_index is None:
                    self._boilerplate_index = len(self._result)

                return

        safe_attrs = attrs

        if tag == 'html':
            if 'i-amphtml-layout' in (attr[0] for attr in attrs):
                # A simple check to see if it’s transformed already
                self._is_render_cancelled = True
            else:
                html_attrs = [('i-amphtml-layout', None), ('i-amphtml-no-boilerplate', None)]
                if not self._is_test_mode:
                    html_attrs.append(('transformed', 'self;v=1'))

                safe_attrs.extend(html_attrs)

        if tag in ['template', 'script']:
            # Don't render amp-* elements inside of `template` or `script`
            self._is_render_paused = True

            if tag == 'script':
                for attr in attrs:
                    if attr[0] == 'custom-element' and attr[1] in self.RENDER_DELAYING_EXTENSIONS:
                        """Don’t remove boilerplate if one of the render-
                        delaying extensions is included"""
                        self._should_remove_boilerplate = False

        sizer = None
        maybe_img_attrs = None
        if tag == 'amp-audio':
            # Don’t remove boilerplate if `amp-audio` is included
            self._should_remove_boilerplate = False

        elif not self._is_render_cancelled and \
                not self._is_render_paused and \
                tag.startswith('amp-') and \
                'data-norender' not in (attr[0] for attr in attrs):

            if tag == 'amp-experiment':
                """Start the finite automata to see if we need to keep the
                boilerplate"""
                self._is_expecting_experiment_script = True

            amp_element = AMPNode(tag, attrs)

            try:
                transformation = amp_element.transform(self._get_next_auto_id())
            except TransformationError:
                self._should_remove_boilerplate = False
            else:
                if transformation:
                    translations, used_auto_id = transformation
                    self._translated_css_data.extend(translations)

                    if used_auto_id:
                        # We had to generate an ID for this element
                        self._increment_auto_id_num()

            safe_attrs = amp_element.get_attrs()
            sizer = amp_element.sizer
            maybe_img_attrs = amp_element.maybe_img_attrs

        # Turn attribute data in to strings
        attr_strings = []
        for attr in safe_attrs:
            if attr[1] is not None:
                value = str(attr[1])
                if self.should_trim_attrs:
                    value = value.strip()
                value = value.replace('"', '&quot;')
                attr_strings.append(' %s="%s"' % (attr[0].lower(), value))
            else:
                attr_strings.append(' %s' % attr[0].lower())

        if self._is_test_mode:
            # Sort alphabetically for diffing
            attr_strings.sort()

        attr_string = ''.join(attr_strings)

        self._result.append('<%s%s>' % (tag, attr_string))

        # Add sizer if necessary
        if sizer:
            sizer_attr_strings = []
            for attr in sizer[0]:
                if attr[1] is not None:
                    value = str(attr[1])
                    value = value.replace('"', '&quot;')
                    sizer_attr_strings.append(' %s="%s"' % (attr[0].lower(), value))
                else:
                    sizer_attr_strings.append(' %s' % attr[0].lower())
            sizer_attr_string = ''.join(sizer_attr_strings)

            self._result.append('<i-amphtml-sizer%s>' % sizer_attr_string)

            if sizer[1] is not None:
                img_attr_strings = []
                for attr in sizer[1]:
                    if attr[1] is not None:
                        value = str(attr[1])
                        value = value.replace('"', '&quot;')
                        img_attr_strings.append(' %s="%s"' % (attr[0].lower(), value))
                    else:
                        img_attr_strings.append(' %s' % attr[0].lower())
                img_attr_string = ''.join(img_attr_strings)

                self._result.append('<img%s>' % img_attr_string)

            self._result.append('</i-amphtml-sizer>')

        # Add img if necessary
        if maybe_img_attrs:
            img_attr_strings = []
            for attr in maybe_img_attrs:
                if attr[1] is not None:
                    value = str(attr[1])
                    value = value.replace('"', '&quot;')
                    img_attr_strings.append(' %s="%s"' % (attr[0].lower(), value))
                else:
                    img_attr_strings.append(' %s' % attr[0].lower())
            img_attr_string = ''.join(img_attr_strings)

            self._result.append('<img%s>' % img_attr_string)

        # Add runtime styles if necessary
        if tag == 'head' and not self._is_render_cancelled:
            if self._is_test_mode:
                """AMP Optimizer uses a stub like this, and then replaces it
                later. Use a stub so we can test against their expected
                output."""
                style = '<style amp-runtime></style>'
            else:
                style = '<style amp-runtime i-amphtml-version="%s">%s</style>' % (
                    self.runtime_version,
                    self.runtime_styles)
            self._result.append(style)

        if tag == 'style':
            """Insert a placeholder into <style amp-custom> so we can add in
            the transformed styles later."""
            if 'amp-custom' in (attr[0] for attr in attrs) and self._translated_styles_index is None:
                self._found_custom_element = True
                self._translated_styles_index = len(self._result)

    def handle_endtag(self, tag):
        tag = tag.lower()

        if self._is_expecting_experiment_data:
            # Finish ingesting JSON data
            data = self._apply_experiment_data()

            if tag == 'script':
                try:
                    # If valid JSON...
                    json_data = json.loads(data)
                except ValueError:
                    pass
                else:
                    # If data wasn’t empty...
                    if json_data:
                        self._is_expecting_experiment_end = True

        elif self._is_expecting_experiment_script:
            self._is_expecting_experiment_script = False

        elif self._is_expecting_experiment_end:
            """If successful experiment and only one child of node, then
            there is an experiment active and the boilerplate can't be
            removed."""
            self._is_expecting_experiment_end = False

            if tag == 'amp-experiment':
                self._should_remove_boilerplate = False

        if tag == 'noscript':
            self._is_in_noscript = False

        if tag == 'style' and self._is_in_boilerplate:
            self._is_in_boilerplate = False
            return

        if tag == 'head' and not self._found_custom_element and self._translated_styles_index is None:
            """If there was no custom element found in the head, add the
            placeholder at the end in case we have custom styles to add later.

            self._found_custom_element will remain False, and we’ll inspect
            that later to decide whether the <script> element itself needs to
            be added."""
            self._translated_styles_index = len(self._result)

        if tag in ['template', 'script']:
            self._is_render_paused = False

        self._result.append('</%s>' % tag)

    def _add_data(self, data):
        if self._is_in_boilerplate:
            if self._is_in_noscript:
                self._noscript_boilerplate += data
                return

            self._boilerplate += data
            return

        if self._is_expecting_experiment_data:
            self._current_experiment_data += data
            return

        self._result.append(data)

    def handle_data(self, data):
        self._add_data(data)

    def handle_entityref(self, name):
        self._add_data('&%s;' % name)

    def handle_charref(self, name):
        self._add_data('&#%s;' % name)

    def handle_comment(self, data):
        if not self.should_strip_comments:
            self._result.append('<!--%s-->' % data)

    def render(self, data):
        self.reset()

        self._auto_id_nums_to_ignore = []

        conflict_ids = re.findall(r"""id=['"]?{}([0-9]+)['"]?""".format(self.ID_PREFIX), data)
        for conflict_id in conflict_ids:
            self._auto_id_nums_to_ignore.append(int(conflict_id))

        self.feed(data)
        self.close()

        # Combine translated styles by media query and value when possible
        media_batches = OrderedDict()

        for selector, statements in self._translated_css_data:
            media_batch_key = tuple(statements.keys())
            batch = media_batches.get(media_batch_key) or OrderedDict()

            for query in statements:
                value_dict = batch.get(query) or OrderedDict()

                value_key = statements[query]
                selector_list = value_dict.get(value_key) or []
                selector_list.append(selector)
                value_dict[value_key] = selector_list

                batch[query] = value_dict

            media_batches[media_batch_key] = batch

        css_parts = []
        for key in media_batches:
            batch = media_batches[key]

            for query in batch:
                parts = []

                for value in batch[query]:
                    selector = ','.join(batch[query][value])
                    parts.append('%s{%s}' % (selector, value))

                css = ''.join(parts)
                if query:
                    css = '@media %s{%s}' % (query, css)

                css_parts.append(css)

        style_string = ''.join(css_parts)

        if style_string and not self._found_custom_element:
            # Insert the amp-custom tag if necessary
            style_string = '<style amp-custom>%s</style>' % style_string

        if self._translated_styles_index is not None:
            self._result.insert(self._translated_styles_index, style_string)

            if (self._boilerplate_index or 0) > self._translated_styles_index:
                self._boilerplate_index += 1

            if (self._noscript_boilerplate_index or 0) > self._translated_styles_index:
                self._noscript_boilerplate_index += 1

        self.no_boilerplate = True
        if self._is_render_cancelled or not self._should_remove_boilerplate:
            self.no_boilerplate = False

            # Restore the boilerplate
            if self._boilerplate_index is not None:
                boilerplate = '<style amp-boilerplate>%s</style>' % self._boilerplate
                self._result.insert(self._boilerplate_index, boilerplate)

                if (self._noscript_boilerplate_index or 0) > self._boilerplate_index:
                    self._noscript_boilerplate_index += 1

            if self._noscript_boilerplate_index is not None:
                noscript_boilerplate = '<style amp-boilerplate>%s</style>' % self._noscript_boilerplate
                self._result.insert(self._noscript_boilerplate_index, noscript_boilerplate)

        result = ''.join(self._result)

        if self._is_render_cancelled or not self._should_remove_boilerplate:
            result = result.replace(' i-amphtml-no-boilerplate', '')

        # Remove empty noscript tags; This happens when removing boilerplate
        result = result.replace('<noscript></noscript>', '')

        return result



# -*- coding: UTF-8 -*-
from __future__ import absolute_import, unicode_literals

# Standard Library
import codecs
import os
import re
import sys
from builtins import bytes  # noqa
from builtins import str  # noqa

# Third Party
import pytest

# AMP Renderer
from amp_renderer import AMPRenderer

if sys.version_info[0] < 3:
    # Third Party
    from HTMLParser import HTMLParser

else:
    # Standard Library
    from html.parser import HTMLParser


class OutputNormalizer(HTMLParser, object):
    """Alphabetize attributes, convert tags to lowercase, etc."""

    def __init__(self, *args, **kwargs):
        super(OutputNormalizer, self).__init__(*args, **kwargs)

        # Always keep charrefs intact; This class is meant to reproduce HTML.
        self.convert_charrefs = False

    def reset(self):
        super(OutputNormalizer, self).reset()

        self._result = ''

    def handle_decl(self, decl):
        self._result = '{}<!{}>'.format(self._result, decl.lower())

    def handle_starttag(self, tag, attrs):
        tag = tag.lower()

        attr_strings = []
        for attr in attrs:
            if attr[1] is not None:
                value = str(attr[1])
                value = value.replace('"', '&quot;')
                attr_strings.append(' {}="{}"'.format(attr[0].lower(), value))
            else:
                attr_strings.append(' {}'.format(attr[0].lower()))

        # Sort alphabetically for diffing
        attr_strings.sort()

        attr_string = ''.join(attr_strings)

        self._result = '{}<{}{}>'.format(self._result, tag, attr_string)

    def handle_endtag(self, tag):
        tag = tag.lower()

        self._result = '{}</{}>'.format(self._result, tag)

    def _add_data(self, data):
        self._result = '{}{}'.format(self._result, data)

    def handle_data(self, data):
        self._add_data(data)

    def handle_entityref(self, name):
        self._add_data('&{};'.format(name))

    def handle_charref(self, name):
        self._add_data('&#{};'.format(name))

    def handle_comment(self, data):
        self._result = '{}<!--{}-->'.format(self._result, data)

    def render(self, data):
        self.reset()
        self.feed(data)
        self.close()

        return self._result


class TestSpec:
    TESTS = [
        'adds_i_amphtml_layout_attribute',
        'boilerplate_not_removed_when_amp_experiment_present',
        'boilerplate_not_removed_when_amp-story_present',
        'boilerplate_not_removed_when_amp_experiment_present',
        'boilerplate_removed_when_amp_experiment_present_but_empty',
        'boilerplate_removed_when_amp_experiment_present_but_invalid_json',
        'boilerplate_removed_when_amp_experiment_present_but_no_tag',
        'boilerplate_then_noscript_removed',
        'converts_heights_attribute_to_css',
        'converts_media_attribute_to_css',
        'converts_sizes_attribute_to_css',
        'does_not_appy_transformations_if_already_present',
        'does_not_break_noscript_tags',
        'does_not_change_content_in_templates',
        'does_not_transform_amp_audio',
        'does_not_transform_invalid_measurements',
        'empty_custom_styles',
        'noscript_then_boilerplate_removed',
        'removes_amp_boilerplate',
        'transforms_layout_container',
        'transforms_layout_fill',
        'transforms_layout_fixed',
        'transforms_layout_fixed_height',
        'transforms_layout_flex_item_with_height',
        'transforms_layout_flex_item_with_width',
        'transforms_layout_flex_item_with_width_and_height',
        'transforms_layout_nodisplay',
        'transforms_layout_responsive',
    ]

    def _format(self, html_value):
        """Remove spaces between tags, and collapse whitespace.

        Far from safe, but fine for tests.
        """
        # remove space between tags
        # https://docs.djangoproject.com/en/3.0/_modules/django/utils/html/
        html_value = re.sub(r'>\s+<', '><', str(html_value))

        # Collapse all whitespace to a single space
        html_value = re.sub(r'\s+', ' ', str(html_value))

        # Add spacing for json
        html_value = re.sub('>{', '> {', str(html_value))
        html_value = re.sub('}</', '} </', str(html_value))
        html_value = re.sub('}{', '} {', str(html_value))
        html_value = re.sub('{}', '{ }', str(html_value))

        return html_value.strip()

    def _run_test(self, spec):
        local_path = os.path.dirname(__file__)
        input_path = '{}/spec/{}/input.html'.format(local_path, spec)
        output_path = '{}/spec/{}/expected_output.html'.format(local_path, spec)

        with codecs.open(input_path, 'r', encoding='utf-8') as html_file:
            html = html_file.read()

        with codecs.open(output_path, 'r', encoding='utf-8') as html_file:
            expected_output = html_file.read()

        normalizer = OutputNormalizer()
        expected = normalizer.render(expected_output)

        # Boot renderer in test mode to match the AMP Optimizer spec
        renderer = AMPRenderer(runtime_version=None, runtime_styles=None)
        result = renderer.render(html)

        assert self._format(result) == self._format(expected)

    @pytest.mark.parametrize('spec', TESTS)
    def test_files(self, spec):
        self._run_test(spec)



# -*- coding: UTF-8 -*-
import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='amp-renderer',
    version='1.4.2',
    author='Chase Finch',
    author_email='chase@finch.email',
    description='Unofficial Python port of server-side rendering from AMP Optimizer',
    keywords=['AMP', 'AMP Optimizer', 'server-side rendering'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/chasefinch/amp-renderer',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=2.7',
    install_requires=[
        'enum34;python_version<"3.4"',
        'future;python_version<="2.7"',
    ],
)



# -*- coding: UTF-8 -*-
from __future__ import absolute_import, unicode_literals

# Standard Library
from builtins import bytes  # noqa
from builtins import str  # noqa

# AMP Renderer
from amp_renderer import AMPRenderer

RUNTIME_VERSION = '01234'
RUNTIME_STYLES = 'body{background-color:pink;}'


class RendererFactory:
    @classmethod
    def make(cls, should_trim_attrs=False, should_strip_comments=False):
        renderer = AMPRenderer(
            runtime_version=RUNTIME_VERSION,
            runtime_styles=RUNTIME_STYLES)

        renderer.should_trim_attrs = should_trim_attrs
        renderer.should_strip_comments = should_strip_comments

        return renderer


# content of test_class.py
class TestRenderer:
    def test_basic(self):
        basic_html = """
            <!doctype html>
            <html ⚡️>
            <head>

            </head>
            <body>
                <div
                    id="testDiv"
                    class="testClass"
                ></div>
            </body>
            </html>
        """

        expected_result = """
            <!doctype html>
            <html ⚡️ i-amphtml-layout i-amphtml-no-boilerplate transformed="self;v=1">
            <head><style amp-runtime i-amphtml-version="{version}">{styles}</style>

            </head>
            <body>
                <div id="testDiv" class="testClass"></div>
            </body>
            </html>
        """.format(version=RUNTIME_VERSION, styles=RUNTIME_STYLES)

        renderer = RendererFactory.make()
        result = renderer.render(basic_html)

        assert result == expected_result
        assert renderer.no_boilerplate

    def test_trim_attributes(self):
        html = """
            <div
                data-test-attribute=" Lovely!  "
                [text]="
                    myFavorites.color
                "
            >
                Blue
            </div>
        """

        expected_result = """
            <div data-test-attribute="Lovely!" [text]="myFavorites.color">
                Blue
            </div>
        """

        renderer = RendererFactory.make(should_trim_attrs=True)
        result = renderer.render(html)

        assert result == expected_result
        assert renderer.no_boilerplate

    def test_strip_comments(self):
        html = '<div><!-- This isn’t important. -->Hello there!</div>'
        expected_result = '<div>Hello there!</div>'

        renderer = RendererFactory.make(should_strip_comments=True)
        result = renderer.render(html)

        assert result == expected_result
        assert renderer.no_boilerplate
